/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.agent;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.xmloperator.reactive.exception.ReactiveException;

import org.xmloperator.reactive.Event;
//import org.xmloperator.reactive.LaunchingContext;
import org.xmloperator.reactive.Message;
import org.xmloperator.reactive.Terms;
import org.xmloperator.reactive.CertifiedAttachment;
import org.xmloperator.reactive.TermsAttachment;
import org.xmloperator.reactive.ActionAttachment;

import org.xmloperator.reactive.facet.Facet;
import org.xmloperator.reactive.facet.FacetImpl;

/**
 * Agent with facets.
 * The facets process only the sender certified messages.
 */
public abstract class FacetsAgent extends AgentImpl {

	protected final List facetList = new ArrayList();
	protected final Map fromServiceToFacet = new HashMap();
	
	/**
	 * Adds a Facet.
	 * A Facet can be added at init or at start-of-instant.
	 * @param facet a Facet.
	 */
	public void addFacet(Facet facet) {
		if (facet == null)
			throw new IllegalArgumentException();
		if (this.launchingContext == null)
			throw new IllegalStateException();
		Object service = facet.getService();
		if (service != null
			&& (this.fromServiceToFacet.put(service, facet) != null))
			throw new ReactiveException(
	"A service cannot be delivered by more than one facet of the same agent");
		this.facetList.add(facet);
		((FacetImpl)facet).attach(this);
		facet.init(this.launchingContext);
	}
	
	/**
	 * Removes a Facet.
	 * A Facet can be removed at start-of-instant.
	 * @param facet
	 */
	public void removeFacet(Facet facet) {
		if (this.facetList.contains(facet)) {
			facet.terminate();
			this.facetList.remove(facet);
		}
	}

	///////////////////////
	// Agent implementation
	///////////////////////
	
	public int idlePeriodDuration_ms() {
		for (int k = 0; k < this.facetList.size(); k++) {
			Facet facet = (Facet)this.facetList.get(k);
			facet.startOfInstant();
		}
		return super.idlePeriodDuration_ms();
	}
	
	public boolean processEvent(Event event) {
		boolean isProcessed = false;
		for (int k = 0; k < this.facetList.size(); k++) {
			Facet facet = (Facet)this.facetList.get(k);
			if (facet.processEvent(event))
				isProcessed = true;
		}
		return isProcessed;
	}

	public boolean processMessage(Message message, Object attachment) {
		if (!(attachment instanceof CertifiedAttachment))
			return false;
		CertifiedAttachment certifiedAttachment =
			(CertifiedAttachment)attachment;
		if (certifiedAttachment.isTermsAttachment()) {
			TermsAttachment termsAttachment = (TermsAttachment)attachment;
			Terms terms = termsAttachment.getTerms();
			Facet processingFacet = null;
			if (terms.count() >= 1) {
				Object service = terms.get(0);
				if (service != null) {
					Facet facet = (Facet)this.fromServiceToFacet.get(service);
					if (facet != null
							&& facet.processServiceMessage(
									message.getName(), termsAttachment))
						processingFacet = facet;
				}
			}
			for (int k = 0; k < this.facetList.size(); k++) {
				Facet facet = (Facet)this.facetList.get(k);
				if (facet != processingFacet)
					facet.processServiceExcludedMessage(
							message, termsAttachment);
			}
			return processingFacet != null;
		}
		else if (certifiedAttachment.isActionAttachment()) {
			ActionAttachment actionAttachment =
				(ActionAttachment)certifiedAttachment;
			for (int k = 0; k < this.facetList.size(); k++) {
				Facet facet = (Facet)this.facetList.get(k);
				facet.processActionMessage(message, actionAttachment);
			}
			return false;
		}
		return false;
	}

	public void terminate() {
		for (int k = 0; k < this.facetList.size(); k++) {
			Facet facet = (Facet)this.facetList.get(k);
			facet.terminate();
		}
	}
}
