/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.element.action.reject;

import org.xmloperator.reactive.Action;

import org.xmloperator.reactive.element.Element;
import org.xmloperator.reactive.element.ParentElement;

import org.xmloperator.reactive.element.impl.ElementImpl;
import org.xmloperator.reactive.element.impl.ParentElementImpl;

import org.xmloperator.reactive.element.spec.ElementSpecification;

import org.xmloperator.reactive.element.action.ElementActionImpl;

/**
 * Rejector of an Action that would touch a specified Element.
 */
public final class ElementTouchRejector implements ActionRejector {
	
	/**
	 * This specify rejection of adding or removing a child to the Element.
	 */
	public static final String REJECT_MODIFY = "modify".intern();
	
	/**
	 * This specify rejection of removing the Element.
	 */
	public static final String REJECT_REMOVE = "remove".intern();
	
	/**
	 * Returns a new ElementTouchRejector with the REJECT_MODIFY type.
	 * @param elementSpecification the specification of the Element
	 *   whose modification has to be rejected.
	 * @return a new ElementTouchRejector with the REJECT_MODIFY type.
	 */
	public static final ElementTouchRejector newModifyRejector(
			ElementSpecification elementSpecification) {
		return new ElementTouchRejector(elementSpecification, REJECT_MODIFY);
	}
	
	/**
	 * Returns a new ElementTouchRejector with the REJECT_REMOVE type.
	 * @param elementSpecification the specification of the Element
	 *   whose removing has to be rejected.
	 * @return a new ElementTouchRejector with the REJECT_REMOVE type.
	 */
	public static final ElementTouchRejector newRemoveRejector(
			ElementSpecification elementSpecification) {
		return new ElementTouchRejector(elementSpecification, REJECT_REMOVE);
	}
	
	public final ElementSpecification elementSpecification;
	public final String rejectionType;
	
	/**
	 * Constructor.
	 * @param elementSpecification the specification of the Element
	 *   whose modification has to be rejected.
	 * @param rejectionType the type of rejection.
	 *   This is REJECT_MODIFY or REJECT_REMOVE.
	 */
	ElementTouchRejector(
			ElementSpecification elementSpecification,
			String rejectionType) {
		if (elementSpecification == null
				|| (rejectionType != REJECT_MODIFY
					&& rejectionType != REJECT_REMOVE))
			throw new IllegalArgumentException();
		this.elementSpecification = elementSpecification;
		this.rejectionType = rejectionType;
	}
	
	///////////////////////////////////////
	// ElementActionRejector implementation
	///////////////////////////////////////
	
	public boolean rejects(Action action) {
		if (!(action instanceof ElementActionImpl))
			return false;
		ElementActionImpl elementAction = (ElementActionImpl)action;
		if (this.rejectionType == REJECT_MODIFY) {
			if (elementAction.isElementInsertAction()
					|| elementAction.isElementRemoveAction())
				return this.elementSpecification.accepts(
					elementAction.referenceElement.getParent());
			if (elementAction.isElementAppendAction()
					|| elementAction.isChildrenRemoveAction()
					|| elementAction.isChildrenSortAction())
				return this.elementSpecification.accepts(
					elementAction.referenceElement);
		}
		else if (this.rejectionType == REJECT_REMOVE) {
			if (elementAction.isElementRemoveAction())
				return this.elementSpecification.accepts(
					elementAction.referenceElement);
			if (elementAction.isChildrenRemoveAction()) {
				Element element =
					((ParentElement)elementAction.referenceElement)
					.getFirstChild();
				while (element != null) {
					if (this.elementSpecification.accepts(element))
						return true;
					element = element.getNextSibling();
				}
			}
		}
		return false;
	}
	
	////////////////////////////////////
	// TranslatableObject implementation
	////////////////////////////////////

	public ParentElement translate() {
		return new ParentElementImpl(Element.REACTIVE_NAMESPACE, "reject",
				new Element[] {
					new ElementImpl(
						Element.REACTIVE_NAMESPACE, this.rejectionType),
					this.elementSpecification.translate()});
	}

	public Class getRebuilderClass() {
		return ElementTouchRejectorRebuilder.class;
	}
}
