/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.facet;

import org.xmloperator.reactive.ActionAttachment;
import org.xmloperator.reactive.Agent;
import org.xmloperator.reactive.Event;
import org.xmloperator.reactive.LaunchingContext;
import org.xmloperator.reactive.Message;
import org.xmloperator.reactive.TermsAttachment;

/**
 * An Facet supports a part of the behaviour of an Agent.
 */
public interface Facet {

	/**
	 * Returns the Agent to which this facet belongs.
	 * It is null if the this facet is not yet attached to an Agent.
	 * @return the Agent to which this facet belongs.
	 */
	public Agent getAgent();
	
	/**
	 * Returns the descriptor (usually a name) of the service delivered
	 *   by this Facet, if any, null elsewhere.
	 * About the usage of the service descriptor,
	 *   see the processServiceMessage() method.
	 * @return the name of the service delivered by this Facet. Can be null.
	 */
	public Object getService();
	
	/**
	 * Executes the initialisation script.
	 * Facets initialisation occurs durint Agent initialisation.
	 * @param launchingContext an inherited context for the Agent.
	 *   See the predefined arguments at the Agent interface.
	 */	
	public void init(LaunchingContext launchingContext);
	
	/**
	 * Notifies a start of instant.
	 */
	public void startOfInstant();

	/**
	 * Processes the given Event and returns true.
	 * Returns false if the Event is unknown.
	 * @param event an Event to process.
	 * @return true if processed, false elsewhere.
	 */
	public boolean processEvent(Event event);

	/**
	 * Processes the given Message and its attachment and returns true.
	 * Returns false if the Message or the attachment content is unknown.
	 * @param messageName the name of a Message to process.
	 * @param attachment the terms of an attachment to the Message.
	 * Pre-condition : the first term of the attachment is the facet
	 *   service descriptor (as returned by the getService() method).
	 * @return true if processed, false elsewhere.
	 */
	public boolean processServiceMessage(
			String messageName, TermsAttachment attachment);

	/**
	 * Processes the given Message and its attachment.
	 * @param message a Message to process.
	 * @param attachment the terms of an attachment to the Message.
	 * Pre-condition : the first term of the attachment, if any, is not
	 *   the facet service descriptor (as returned by the getService() method).
	 */
	public void processServiceExcludedMessage(
			Message message, TermsAttachment attachment);
	
	/**
	 * Processes the given Action Message and its attachment.
	 * @param message an Action Message to process.
	 * @param attachment an ActionAttachment.
	 */
	public void processActionMessage(
			Message message, ActionAttachment attachment);
	
	/**
	 * Terminates the facet.
	 * Facets termination occurs during Agent termination.
	 * Usually owned events are unvalidated here.
	 */
	public void terminate();
}
