/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.impl;

import java.io.PrintStream;

import org.xmloperator.reactive.Agent;
import org.xmloperator.reactive.CrashingReport;
import org.xmloperator.reactive.Event;
import org.xmloperator.reactive.LaunchingContext;
import org.xmloperator.reactive.ReactiveEntity;

final class AgentThread extends Thread {
	
	private static PrintStream trace = null;
	private static boolean isFullTrace = false;
	
	static void setTrace(PrintStream trace, boolean isFullTrace) {
		AgentThread.trace = trace;
		AgentThread.isFullTrace = isFullTrace;
	}

	private final ReactiveMachineImpl machine;
	private final Agent agent;
	private final String agentProfile;
	private final LaunchingContext launchingContext;
	private final CrashingReport report;
	
	public AgentThread(
			ReactiveMachineImpl machine,
			Agent agent,
			String agentProfile,
			LaunchingContext launchingContext,
			CrashingReport report) {
		super("agent");
		if (machine == null || agent == null || agentProfile == null
				|| launchingContext == null)
			throw new IllegalArgumentException();
		this.machine = machine;
		this.agent = agent;
		this.agentProfile = agentProfile;
		this.launchingContext = launchingContext;
		this.report = report;
	}
	
	/**
	 * Process events while the method <code>continues()</code> return true.
	 * @param reac the ReactiveEntity attached to this Agent.
	 */
	private void participateToExchanges(ReactiveEntity reac) {
		while (this.agent.continues()) {
			if (trace != null && isFullTrace)
				trace.println(reac.getEntityId() + " continues");
			if (reac.waitEvent(this.agent.idlePeriodDuration_ms()))
				this.agent.endOfIdlePeriod();
			Event[] events;
			do {
				events = reac.suspend();
				if (events != null)
					this.processEvents(reac, events);
			}
			while (events != null);
		}
	}
	
	private final void processEvents(
			ReactiveEntity reac, Event[] events) {
		for (int ke = 0; ke < events.length; ke++) {
			Event event = events[ke];
			if (event.isMessage()) {
				MessageImpl message = (MessageImpl)event;
				Object[] attachments = message.getAttachments();
				for (int ka = 0; ka < attachments.length; ka++) {
					Object attachment = attachments[ka];
					if (this.agent.processMessage(message, attachment)
							&& trace != null)
						trace.println(reac.getEntityId() + " has processed "
							+ message + " " + attachment);
				}
			}
			else {
				if (this.agent.processEvent(event) && trace != null)
					trace.println(reac.getEntityId() + " has processed " + event);
			}
		}
	}
	
	private final void crash(String entityId, Exception exception) {
		if (trace != null)
			trace.println(entityId + " crash");
		if (this.report != null)
			this.report.notifyCrash(entityId, exception);
		else
			exception.printStackTrace();
	}

	////////////////////////
	// Thread implementation
	////////////////////////
	
	public void run() {
		ReactiveEntityImpl reac =
			(ReactiveEntityImpl)this.launchingContext.getArgumentValue(
				Agent.ARG_REACTIVE_ENTITY); 
		reac.setThread(this);
		if (trace != null)
			trace.println(reac + " is " + this.agent.getAgentId());
		String entityId = reac.getEntityId();
		this.setName(entityId);
		try {
			ReactiveCommunityImpl community =
				(ReactiveCommunityImpl)launchingContext.getArgumentValue(
					Agent.ARG_DESTINATION_COMMUNITY);
			if (community != null) {
				community.connect(reac, this.machine);
				/*
				 * Connection acknowledgment calls notifyAgentStarting().
				 */
				reac.setCommunity(community);
			}
			else
				this.machine.notifyAgentStarting(entityId);
			try {
				/*
				 * Agent code dependent part.
				 */
				if (trace != null && isFullTrace)
					trace.println(entityId + " init");
				this.agent.init(this.launchingContext);
				try {
					this.participateToExchanges(reac);
				}
				catch (Exception exception) {
					this.crash(entityId, exception);
				}
				if (trace != null && isFullTrace)
					trace.println(entityId + " terminate");
				this.agent.terminate();
			}
			catch (Exception exception) {
				this.crash(entityId, exception);
			}
			reac.disconnect();
			this.machine.notifyAgentTermination(entityId);
		}
		catch (Exception exception) {
			this.crash(entityId, exception);
		}
	}
}
