/*
 * Copyright (c) 2008 The_xmloperator_project.  All rights reserved.
 * See terms of use on the project's site (http://www.xmloperator.net/reactive/).
 */
package org.xmloperator.reactive.test;

import org.xmloperator.reactive.Agent;
import org.xmloperator.reactive.LaunchingContext;
import org.xmloperator.reactive.Message;
import org.xmloperator.reactive.ReactiveCommunity;

import org.xmloperator.reactive.agent.AgentImpl;
import org.xmloperator.reactive.util.End;
import org.xmloperator.reactive.util.TermsAttachmentFactory;

/**
 * The purpose of this SyracuseAgent consists in stressing a ReactiveMachine
 *   about the Syracuse conjecture.
 * The SyracuseAgent recursively launches new instances of itself.
 */
public final class SyracuseAgent extends AgentImpl {
	
	private static final String START = "start".intern();
	
	private boolean isMain;
	private final long start;
	private final long target;
	
	/**
	 * The ExitFacet is not usable here since new Agent instances are
	 *   perpetually launched.
	 */
	private final End globalEnd;
	
	private long current = 0;
	private long endInstant = Long.MAX_VALUE;

	/**
	 * The Syracuse recurence is :
	 *   if n is odd then n -> (3n + 1)/2,
	 *   if n is even then n -> n/2.
	 * The SyracuseAgent does the opposite :
	 *   if n % 3 == 2 then n -> (2n - 1)/3,
	 *   if n % 3 != 0 then n -> 2n.
	 * @param isMain is true for the main Agent, false for the other ones.
	 * @param start a starting strictly positive odd integer.
	 * @param target an odd and strictly positive target that terminates
	 *   the processus once obtained.
	 * @param globalEnd a synchronized end.
	 */
	public SyracuseAgent(boolean isMain, long start, long target,
			End globalEnd) {
		if (start < 1 || start % 2 != 1 || target < 1 || target % 2 != 1
				|| globalEnd == null)
			throw new IllegalArgumentException();
		this.isMain = isMain;
		this.start = start;
		this.target = target;
		this.globalEnd = globalEnd;
		this.addParameter(start);
	}
	
	protected ReactiveCommunity newCommunity() {
		return this.reac.newCommunity("SyracuseConjecture");
	}
	
	///////////////////////
	// Agent implementation
	///////////////////////
	
	/**
	 * Are deterministic :
	 *   the instant count,
	 *   the set of numbers generated within each instant.
	 * Are not deterministic :
	 *   the order of Agent instanciation,
	 *   the relation between entityIds and represented numbers.
	 */
	public void init(LaunchingContext launchingContext) {
		super.init(launchingContext);
		Message m_start = this.reac.searchMessage(true, START);
		this.reac.generateMessage(m_start,
				TermsAttachmentFactory.newTermsAttachment(
						new Long(this.start)));
		long instant = this.reac.getCommunity().getInstant();
		if (this.start == this.target) {
			this.endInstant = instant;
			this.globalEnd.endInstant = instant + 1;
		}
		else if (this.start == 1)
			this.current = 8;
		else {
			long start_mod_3 = this.start % 3; 
			if (start_mod_3 == 0)
				this.endInstant = instant;
			else {
				this.current = this.start;
				if (start_mod_3 == 1)
					this.current *= 2;
			}
		}
	}

	public boolean continues() {
		long instant = this.reac.getCommunity().getInstant();
		if (instant >= this.endInstant || instant >= this.globalEnd.endInstant)
			return false;
		{
			long newStart = (2 * this.current - 1) / 3;
			Agent newAgent =
				new SyracuseAgent(false, newStart, this.target,
					this.globalEnd);
			this.reac.launchAgent(newAgent, null);
		}
		this.current *= 4;
		return true;
	}

	public void terminate() {
		super.terminate();
		if (this.isMain)
			this.reac.closeCommunity();
	}
}
